from flask import Flask, request, render_template, url_for
import requests
import json
import urllib
import pprint
import codecs

app = Flask(__name__)

@app.route("/", methods=['GET', 'POST'])
def index():
    #url = 'https://fhirtestjp.med.gunma-u.ac.jp/Patient?params=/'
    url = 'https://hapi.fhir.org/baseR4/Patient?identifier='

    # 患者IDからPatientリソースを検索する
    if request.method == 'POST':
        # 入力されたPatient IDからURLを生成
        identifier_ = request.form['identifier_']
        url += identifier_

        # URLからJSONを取得
        with urllib.request.urlopen(url) as response:
            fhir = response.read()
            # jsonから辞書型に変換
            text = json.loads(fhir.decode('utf-8'))
        
        # 患者IDが見つからなかった場合は住所情報なし
        if text.get('total') > 0:
            note = '患者ID' + identifier_ + '　の住所情報が見つかりました'
            address = text['entry'][-1]['resource']['address'][0]['text']
            postalCode = text['entry'][-1]['resource']['address'][0]['postalCode']
        
        # 患者IDが見つかった場合は住所情報を取得する
        else:
            note = '患者ID' + identifier_ + '　の住所情報はありません'
            address = ''
            postalCode = ''
        
        # FHIR取得したリソースを「text」という変数に入れhtmlに送る
        return render_template("index_get.html", identifier_=identifier_, text=text, note=note, address=address, postalCode=postalCode)
        
    else: # GET
        return render_template("index_get.html")

@app.route("/post", methods=['GET', 'POST'])
def index2():
    #url = 'https://fhirtestjp.med.gunma-u.ac.jp/Patient?params=/'
    url = 'http://hapi.fhir.org/baseR4/Patient?identifier='

    identifier_ = request.form['identifier_']
    url += identifier_
        
    # URLからJSONを取得
    with urllib.request.urlopen(url) as response:
        fhir = response.read()
        # jsonから辞書型に変換
        text = json.loads(fhir.decode('utf-8'))
    
    # 住所情報の部分をフォームの内容に書き換える
    address = request.form['address']
    postalCode = request.form['postalCode']
    text['entry'][-1]['resource']['address'][0]['text'] = address
    text['entry'][-1]['resource']['address'][0]['postalCode'] = postalCode
    text = text['entry'][-1]['resource']
    add_json = json.dumps(text)
    
    # 書き換えられた情報をPOSTする
    url = 'http://hapi.fhir.org/baseR4/Patient?identifier='
    identifier_ = request.form['identifier_']
    url += identifier_

    headers = {"content-type": "application/json"}
    response = requests.post(url=url, headers=headers, data=add_json)
    response.encoding = response.apparent_encoding
    print(response.status_code)
    print(response.text)

    note = '患者ID' + identifier_ + '　の住所情報を変更しました。'

    #jsonデータをテキストファイルに出力する
    path_w = './app21out.txt' # ※ここのパスは好きな場所に書き換え可
    with open(path_w, mode='w') as f:
        f.write(add_json) # 問題なく出力されることを確認します。
    
    return render_template("index_post.html", identifier_=identifier_, text=text, note=note, add=address, postalCode=postalCode)

if __name__ == '__main__':
    app.run(debug=True)