#!/usr/bin/env python3

# ライブラリのインポート
import datetime
import requests
from requests.auth import HTTPBasicAuth
import json
import pprint

# -- 食物アレルギー情報を生成する
def createFoodAllergyIntolerance(patientId):
    allgr=dict()

    allgrcd='J9FB13110000'
    allgrname='鶏卵'
    reactcd='247472004'
    reactname='蕁麻疹'
    reactlevel='mild'
    critical='low'
    occrdatetime=datetime.datetime(2023,6,20,8,10,15)
    recdatetime=datetime.datetime(2023,6,20,10,30,45)
    recstaffid='1853'

    allgr['resourceType']='AllergyIntolerance'
    allgr['id']=f'{patientId}-{allgrcd}'

    meta=dict()
    meta['profile']=['http://jpfhir.jp/fhir/core/StructureDefinition/JP_AllergyIntolerance']
    allgr['meta']=meta

    pat=dict()
    pat['reference']=f'Patient/{patientId}'
    allgr['patient']=pat

    allgr['type']='allergy'
    allgr['category']=['food']

    algcd=dict()
    algcd['system']='http://jpfhir.jp/fhir/core/CodeSystem/JP_JfagyFoodAllergen_CS'
    algcd['code']=allgrcd
    algcd['display']=allgrname
    algitm=dict()
    algitm['coding']=[algcd]
    algitm['text']=allgrname
    allgr['code']=algitm

    allgr['onsetDateTime']=occrdatetime.strftime('%Y-%m-%dT%H:%M:%S') + '+09:00'

    manifes=dict()
    manifescd=dict()
    manifescd['system']='http://snomed.info/sct'
    manifescd['code']=reactcd
    manifescd['display']=reactname
    manifes=dict()
    manifes['coding']=[manifescd]
    manifes['text']=reactname
    reactitm=dict()
    reactitm['manifestation']=[manifes]
    reactitm['severity']=reactlevel
    allgr['reaction']=[reactitm]

    allgr['criticality']=critical

    clstscd=dict()
    clstscd['system']='http://terminology.hl7.org/CodeSystem/allergyintolerance-clinical'
    clstscd['code']='active'
    clsts=dict()
    clsts['coding']=[clstscd]
    allgr['clinicalStatus']=clsts

    vrfstscd=dict()
    vrfstscd['system']='http://terminology.hl7.org/CodeSystem/allergyintolerance-verification'
    vrfstscd['code']='confirmed'
    vrfsts=dict()
    vrfsts['coding']=[vrfstscd]
    allgr['verificationStatus']=vrfsts

    staff=dict()
    staff['reference']=f'Practitioner/{recstaffid}'
    allgr['recorder']=staff
    allgr['recordedDate']=recdatetime.strftime('%Y-%m-%dT%H:%M:%S') + '+09:00'

    return allgr

# -- 薬剤アレルギー情報を生成する
def createMedicationAllergyIntolerance(patientId):
    allgr=dict()

    allgrcd='YCM1214401A1027'
    allgrname='キシロカイン注射液０．５％'
    reactcd='422400008'
    reactname='嘔吐'
    reactlevel='mild'
    critical='low'
    occrdatetime=datetime.datetime(2023,6,21,8,10,15)
    recdatetime=datetime.datetime(2023,6,21,10,30,45)
    recstaffid='1853'

    # ここに食物アレルギー情報のリソース作成を参考に薬剤アレルギー情報のリソース作成処理を追加しましょう
    # https://jpfhir.jp/fhir/core/1.1.1/StructureDefinition-jp-allergyintolerance.html

    return allgr

# -- 指定した患者のアレルギー情報を保存する
def saveAllergyIntolerance(allgr):
#    resturl = f'http://hapi.fhir.org/baseR4/AllergyIntolerance'
    resturl = f'http://20.78.56.128:52773/csp/healthshare/fhirserver/fhir/r4/AllergyIntolerance'

#    headers = {"content-type": "application/json"}
    headers = {"content-type": "application/json+fhir","Accept-Charset":"utf8"}
    basic_auth=HTTPBasicAuth("_SYSTEM", "sys")

    response = requests.post(url=resturl, headers=headers,json=allgr,auth=basic_auth)
    
    #jsonobj = response.json()
    #pprint.pprint(jsonobj)

# -- メイン処理

print('-- 食物アレルギー情報登録 --')
allgr=createFoodAllergyIntolerance('2')
pprint.pprint(allgr)

path_w='AllergyIntolerance-1.json'
with open(path_w, mode='w',encoding="utf-8") as f:
   jsonstr = json.dumps(allgr, indent=2, ensure_ascii=False)
   f.write(jsonstr)

saveAllergyIntolerance(allgr)

print('-- 薬剤アレルギー情報登録 --')
allgr=createMedicationAllergyIntolerance('2')
pprint.pprint(allgr)

path_w='AllergyIntolerance-2.json'
with open(path_w, mode='w',encoding="utf-8") as f:
   jsonstr = json.dumps(allgr, indent=2, ensure_ascii=False)
   f.write(jsonstr)

saveAllergyIntolerance(allgr)

