
import requests
from requests.auth import HTTPBasicAuth
import datetime
import json
import re
import pandas as pd
from flask import Flask , request, render_template, Markup, send_file

app = Flask(__name__)

def loadAllergyIntolerance(patientId):
    """
    FHIRサーバから患者のアレルギー情報を読み込む

    Parameters
    ----------
    patientId: string
        検索する患者ID

    Returns
    -------
    jsonobj: dict
        アレルギー情報のJSONデータを格納したディクショナリ
    """
    #resturl = f'https://fhirtestjp.med.gunma-u.ac.jp/Bundle?{query_text}'
    #resturl = f'http://hapi.fhir.org/baseR4/AllergyIntolerance?patient={patientId}'
    resturl = f'http://20.78.56.128:52773/csp/healthshare/fhirserver/fhir/r4/AllergyIntolerance?patient={patientId}'

    #headers = {"content-type": "application/json"}
    headers = {"content-type": "application/json+fhir","Accept-Charset":"utf8"}
    basic_auth=HTTPBasicAuth("_SYSTEM", "sys")

    response = requests.get(url=resturl, headers=headers,auth=basic_auth)

    jsonobj = response.json()

    return jsonobj

@app.route('/')
def index():
    '''
    アレルギー情報を読み込む患者の検索画面を表示する
    '''
    return render_template('index.html')

@app.route('/AllergyViewer', methods=["POST"])
def viewSummary():
    '''
    指定したパラメータの条件でFHIRサーバのアレルギー情報を検索し画面に表示する
    '''
    ptid=request.form["patientId"] 

    jsondoc=loadAllergyIntolerance(ptid)

    algrlst=json.loads('[]')
    if jsondoc['total'] == 0:
        print(f'ID:{jsondoc}のアレルギー情報なし')
    else:
        for itm in jsondoc['entry']:
            algrlst.append(itm['resource'])

    return render_template('allergy.html',patientId=ptid,allergyList=algrlst)


if __name__ == "__main__":
    app.run(debug=True)

