#!/usr/bin/env python3

# ライブラリのインポート
import json
import re
import pandas as pd
from openpyxl import Workbook
from xhtml2pdf import pisa

# -- app1.pyで取得したJSONファイルを読み込みExcelファイルに出力

## JSONファイルを読み込んで辞書型に変換

jsondoc=None
json_path = './summary.json'
with open(json_path, mode='r',encoding="utf-8") as json_file:
    jsonstr=json_file.read()
    jsondoc = json.loads(jsonstr)

## Excelデータを新規に作成して書き込み準備
xlsx_wb = Workbook()
xlsx_ws = xlsx_wb['Sheet']
xlsx_ws = xlsx_wb.active
print(f'sheet name: {xlsx_ws.title}')

## 退院時サマリーデータのうち文書情報を格納したCompositionリソースを読み込んで、
## 基本情報の取得と他のリソースへの参照を取得

lst=[p for p in jsondoc['entry'] if p['resource']['resourceType']=='Composition']
subjectid=''
encounterid=''
autheridlst=[]
if len(lst)>0 :
    composi=lst[0]['resource']

    xlsx_ws['A1'] = '文書ID'
    print(composi['identifier']['value'])
    xlsx_ws['B1'] = composi['identifier']['value']

    xlsx_ws['A2'] = '文書種類コード'
    print(composi['type']['coding'][0]['code'])
    xlsx_ws['B2'] = composi['type']['coding'][0]['code']

    xlsx_ws['A3'] = '文書種類名'
    print(composi['type']['coding'][0]['display'])
    xlsx_ws['B3'] = composi['type']['coding'][0]['display']

    xlsx_ws['A4'] = '文書タイトル'
    print(composi['title'])
    xlsx_ws['B4'] = composi['title']

    xlsx_ws['A5'] = '文書作成日'
    print(composi['date'])
    xlsx_ws['B5'] = composi['date']

    xlsx_ws['A6'] = '文書状態'
    print(composi['status'])
    xlsx_ws['B6'] = composi['status']

    subjectid=composi['subject']['reference']
    encounterid=composi['encounter']['reference']
    autheridlst=[i['reference'] for i in composi['author']]
else:
    raise NameError('entryにCompositionなし')

## Patientリソースを検索し患者情報を取得

lst=[p for p in jsondoc['entry'] if p.get('fullUrl')==subjectid]
if len(lst)>0 :
    pat=lst[0]['resource']

    xlsx_ws['A8'] = '患者ID'
    print(pat['identifier'][0]['value'])
    xlsx_ws['B8'] = pat['identifier'][0]['value']

    xlsx_ws['A9'] = '患者氏名'
    print(pat['name'][0]['text'])
    xlsx_ws['B9'] = pat['name'][0]['text']

    xlsx_ws['A10'] = '患者カナ'
    print(pat['name'][1]['text'])
    xlsx_ws['B10'] = pat['name'][1]['text']

    xlsx_ws['A11'] = '性別'
    print(pat['gender'])
    xlsx_ws['B11'] = pat['gender']

    xlsx_ws['A12'] = '生年月日'
    print(pat['birthDate'])
    xlsx_ws['B12'] = pat['birthDate']

    xlsx_ws['A13'] = '郵便番号'
    print(pat['address'][0]['postalCode'])
    xlsx_ws['B13'] = pat['address'][0]['postalCode']

    xlsx_ws['A14'] = '住所'
    print(pat['address'][0]['text'])
    xlsx_ws['B14'] = pat['address'][0]['text']
else:
    raise NameError('entryにPatientなし')

## Encounterリソースを検索し入院情報と関連する病名情報を取得

lst=[e for e in jsondoc['entry'] if e.get('fullUrl')==encounterid]
if len(lst)>0 :
    enc=lst[0]['resource']
    
    xlsx_ws['A16'] = '入院日'
    print(enc['period']['start'])
    xlsx_ws['B16'] = enc['period']['start']
    
    xlsx_ws['A17'] = '退院日'
    print(enc['period']['end'])
    xlsx_ws['B17'] = enc['period']['end']

    xlsx_ws['A18'] = '入院理由'
    print(','.join(enc['reasonCode']))
    xlsx_ws['B18'] = ','.join(enc['reasonCode'])

    diaglst=sorted(enc['diagnosis'],key=lambda x: x['rank'])
    adlst=[]
    for diag in diaglst:
        adcd=[i for i in diag['use']['coding'] if i['code']=='AD']
        if(len(adcd)>0):
            adlst.append(diag['condition'])
    for condiid in adlst:
        condilst=[e for e in jsondoc['entry'] if e.get('fullUrl')==condiid]
        if len(condiid)>0:
            condi=condilst[0]['resource']
            xlsx_ws['A19'] = '入院時診断名'
            print(condi['code']['text'])
            xlsx_ws['B19'] = condi['code']['text']

            xlsx_ws['A20'] = '入院時診断日'
            print(condi['onsetDateTime'])
            xlsx_ws['B20'] = condi['onsetDateTime']
    ddlst=[]
    for diag in diaglst:
        ddcd=[i for i in diag['use']['coding'] if i['code']=='DD']
        if(len(ddcd)>0):
            ddlst.append(diag['condition'])
    for condiid in ddlst:
        condilst=[e for e in jsondoc['entry'] if e.get('fullUrl')==condiid]
        if len(condiid)>0:
            condi=condilst[0]['resource']
            xlsx_ws['A21'] = '退院時診断名'
            print(condi['code']['text'])
            xlsx_ws['B21'] = condi['code']['text']

            xlsx_ws['A22'] = '退院時診断日'
            print(condi['onsetDateTime'])
            xlsx_ws['B22'] = condi['onsetDateTime']
else:
    raise NameError('entryにEncounterなし')

## Compositionのauthorには医療機関と医師情報の両方が格納されているので、
## OrganizationとPractitionerリソースを検索し医療機関と医師情報を取得

for autherid in autheridlst:
    lst=[i for i in jsondoc['entry'] if i.get('fullUrl')==autherid and i.get('resource').get('resourceType')=='Organization']
    if len(lst)>0:
        orga=lst[0]['resource']

        xlsx_ws['A24'] = '病院コード'
        print(orga['identifier'][0]['value'])
        xlsx_ws['B24'] = orga['identifier'][0]['value']

        xlsx_ws['A25'] = '病院名'
        print(orga['name'])
        xlsx_ws['B25'] = orga['name']

    lst=[i for i in jsondoc['entry'] if i.get('fullUrl')==autherid and i.get('resource').get('resourceType')=='Practitioner']
    if len(lst)>0:
        pract=lst[0]['resource']

        xlsx_ws['A26'] = '医師ID'
        print(pract['identifier'][0]['value'])
        xlsx_ws['B26'] = pract['identifier'][0]['value']

        xlsx_ws['A27'] = '医師氏名'
        print(pract['name'][0]['text'])
        xlsx_ws['B27'] = pract['name'][0]['text']

        xlsx_ws['A28'] = '医師カナ'
        print(pract['name'][1]['text'])
        xlsx_ws['B28'] = pract['name'][1]['text']

## Excelファイルを書き込む

xlsx_wb.save('summary.xlsx')


# -- ExcelファイルからHTMLに変換

## EXCELファイルを読み込み

excel_df = pd.read_excel("summary.xlsx", engine='openpyxl',header=None)

print(excel_df)
htmllines=[]

## HTMLのヘッダを作成

htmllines.append('<html><head>\n')
htmllines.append('<meta http-equiv="content-type" content="text/html; charset=utf-8">\n')
htmllines.append('<title>summary</title>\n')
htmllines.append('<style> body { font-family: HeiseiKakuGo-W5 } </style>\n')
htmllines.append('</head><body>\n')

## EXCELで表形式に格納された退院時サマリーデータをHTMLのTableとして作成

htmllines.append('<table border="1">\n')
for i in range(len(excel_df)):
    htmllines.append("\t<tr>\n")
    for k in range(len(excel_df.columns)):
        val = excel_df.iloc[i, k]
        rowspan = 1
        if isinstance(val, str):
            val = re.sub("(\r\n)|(\n)", "<br />", val)
            l = 1
            while i+l < len(excel_df):
                _val = excel_df.iloc[i+l, k]
                if isinstance(_val, str):
                    break
                l += 1
            rowspan = l
            htmllines.append(f'\t\t<td colspan="1" rowspan="{rowspan}">{val}</td>\n')
    htmllines.append("\t</tr>\n")
htmllines.append('</table>\n')

## HTMLのフッタを作成

htmllines.append('</body></html>\n')

htmlstr=''.join(htmllines)
print(htmlstr)

## HTMLファイルの書き込み

html_path = 'summary.html'
with open(html_path, mode='w',encoding='UTF-8') as html_file:
    html_file.write(htmlstr)

# -- HTMLファイルからPDFに変換

pdf_path = 'summary.pdf'
with open(pdf_path, "w+b") as pdf_file:
        pisa_status = pisa.CreatePDF(src=htmlstr, dest=pdf_file)
        print(f'Pdf Convert Status:{pisa_status}')
